/*----- PROTECTED REGION ID(Socket2Class.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        Socket2Class.cpp
//
// description : C++ source for the Socket2Class.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the Socket2 once per process.
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
// $Author: alessio $
//
// $Revision: 1.8 $
// $Date: 2018-05-09 16:06:06 $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <Socket2Class.h>

/*----- PROTECTED REGION END -----*/	//	Socket2Class.cpp

//-------------------------------------------------------------------
/**
 *	Create Socket2Class singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_Socket2_class(const char *name) {
		return Socket2_ns::Socket2Class::init(name);
	}
}

namespace Socket2_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
Socket2Class *Socket2Class::_instance = NULL;

//===================================================================
//	Class constants
//===================================================================
//--------------------------------------------------------
/**
 * method : 		Socket2Class::Socket2Class(std::string &s)
 * description : 	constructor for the Socket2Class
 *
 * @param s	The class name
 */
//--------------------------------------------------------
Socket2Class::Socket2Class(std::string &s):Tango::DeviceClass(s)
{
	TANGO_LOG_INFO << "Entering Socket2Class constructor" << std::endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(Socket2Class::constructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::constructor

	TANGO_LOG_INFO << "Leaving Socket2Class constructor" << std::endl;
}

//--------------------------------------------------------
/**
 * method : 		Socket2Class::~Socket2Class()
 * description : 	destructor for the Socket2Class
 */
//--------------------------------------------------------
Socket2Class::~Socket2Class()
{
	/*----- PROTECTED REGION ID(Socket2Class::destructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		Socket2Class::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
Socket2Class *Socket2Class::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			std::string s(name);
			_instance = new Socket2Class(s);
		}
		catch (std::bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		Socket2Class::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
Socket2Class *Socket2Class::instance()
{
	if (_instance == NULL)
	{
		std::cerr << "Class is not initialized !!" << std::endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		WriteClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *WriteClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	TANGO_LOG_INFO << "WriteClass::execute(): arrived" << std::endl;
	const Tango::DevVarCharArray *argin;
	extract(in_any, argin);
	((static_cast<Socket2 *>(device))->write(argin));
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ReadClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ReadClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	TANGO_LOG_INFO << "ReadClass::execute(): arrived" << std::endl;
	Tango::DevLong argin;
	extract(in_any, argin);
	return insert((static_cast<Socket2 *>(device))->read(argin));
}

//--------------------------------------------------------
/**
 * method : 		ReadUntilClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ReadUntilClass::execute(Tango::DeviceImpl *device, const CORBA::Any &in_any)
{
	TANGO_LOG_INFO << "ReadUntilClass::execute(): arrived" << std::endl;
	const Tango::DevVarCharArray *argin;
	extract(in_any, argin);
	return insert((static_cast<Socket2 *>(device))->read_until(argin));
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method     : Socket2Class::get_class_property()
 *	Description: Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum Socket2Class::get_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method     : Socket2Class::get_default_device_property()
 *	Description: Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum Socket2Class::get_default_device_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method     : Socket2Class::get_default_class_property()
 *	Description: Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum Socket2Class::get_default_class_property(std::string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method     : Socket2Class::set_default_property()
 *	Description: Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void Socket2Class::set_default_property()
{
	std::string	prop_name;
	std::string	prop_desc;
	std::string	prop_def;
	std::vector<std::string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "Hostname";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Port";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Protocol";
	prop_desc = "";
	prop_def  = "TCP";
	vect_data.clear();
	vect_data.push_back("TCP");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "Timeout";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "IOMultiplexing";
	prop_desc = "Use `sleep` for fixed sleep and `select` for the syscall of the same name";
	prop_def  = "SELECT";
	vect_data.clear();
	vect_data.push_back("SELECT");
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method     : Socket2Class::write_class_property()
 *	Description: Set class description fields as property in database
 */
//--------------------------------------------------------
void Socket2Class::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	std::string	classname = get_name();
	std::string	header;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	std::string	str_title("");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	std::vector<std::string>	str_desc;
	str_desc.push_back("");
	description << str_desc;
	data.push_back(description);

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	std::vector<std::string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method     : Socket2Class::device_factory()
 *	Description: Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void Socket2Class::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(Socket2Class::device_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		TANGO_LOG_DEBUG << "Device name : " << (*devlist_ptr)[i].in() << std::endl;
		device_list.push_back(new Socket2(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		Socket2 *dev = static_cast<Socket2 *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(Socket2Class::device_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : Socket2Class::attribute_factory()
 *	Description: Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void Socket2Class::attribute_factory(std::vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(Socket2Class::attribute_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::attribute_factory_before
	//	Attribute : InputLength
	InputLengthAttrib	*inputlength = new InputLengthAttrib();
	Tango::UserDefaultAttrProp	inputlength_prop;
	//	description	not set for InputLength
	//	label	not set for InputLength
	//	unit	not set for InputLength
	//	standard_unit	not set for InputLength
	//	display_unit	not set for InputLength
	//	format	not set for InputLength
	//	max_value	not set for InputLength
	//	min_value	not set for InputLength
	//	max_alarm	not set for InputLength
	//	min_alarm	not set for InputLength
	//	max_warning	not set for InputLength
	//	min_warning	not set for InputLength
	//	delta_t	not set for InputLength
	//	delta_val	not set for InputLength
	inputlength->set_default_properties(inputlength_prop);
	//	Not Polled
	inputlength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(inputlength);

	//	Attribute : OutputLength
	OutputLengthAttrib	*outputlength = new OutputLengthAttrib();
	Tango::UserDefaultAttrProp	outputlength_prop;
	//	description	not set for OutputLength
	//	label	not set for OutputLength
	//	unit	not set for OutputLength
	//	standard_unit	not set for OutputLength
	//	display_unit	not set for OutputLength
	//	format	not set for OutputLength
	//	max_value	not set for OutputLength
	//	min_value	not set for OutputLength
	//	max_alarm	not set for OutputLength
	//	min_alarm	not set for OutputLength
	//	max_warning	not set for OutputLength
	//	min_warning	not set for OutputLength
	//	delta_t	not set for OutputLength
	//	delta_val	not set for OutputLength
	outputlength->set_default_properties(outputlength_prop);
	//	Not Polled
	outputlength->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(outputlength);

	//	Attribute : Reconnections
	ReconnectionsAttrib	*reconnections = new ReconnectionsAttrib();
	Tango::UserDefaultAttrProp	reconnections_prop;
	//	description	not set for Reconnections
	//	label	not set for Reconnections
	//	unit	not set for Reconnections
	//	standard_unit	not set for Reconnections
	//	display_unit	not set for Reconnections
	//	format	not set for Reconnections
	//	max_value	not set for Reconnections
	//	min_value	not set for Reconnections
	//	max_alarm	not set for Reconnections
	//	min_alarm	not set for Reconnections
	//	max_warning	not set for Reconnections
	//	min_warning	not set for Reconnections
	//	delta_t	not set for Reconnections
	//	delta_val	not set for Reconnections
	reconnections->set_default_properties(reconnections_prop);
	//	Not Polled
	reconnections->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(reconnections);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(Socket2Class::attribute_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : Socket2Class::pipe_factory()
 *	Description: Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void Socket2Class::pipe_factory()
{
	/*----- PROTECTED REGION ID(Socket2Class::pipe_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::pipe_factory_before
	/*----- PROTECTED REGION ID(Socket2Class::pipe_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method     : Socket2Class::command_factory()
 *	Description: Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void Socket2Class::command_factory()
{
	/*----- PROTECTED REGION ID(Socket2Class::command_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::command_factory_before
	//	Command Write
	WriteClass	*pWriteCmd =
		new WriteClass("Write",
			Tango::DEVVAR_CHARARRAY, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pWriteCmd);

	//	Command Read
	ReadClass	*pReadCmd =
		new ReadClass("Read",
			Tango::DEV_LONG, Tango::DEVVAR_CHARARRAY,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pReadCmd);

	//	Command ReadUntil
	ReadUntilClass	*pReadUntilCmd =
		new ReadUntilClass("ReadUntil",
			Tango::DEVVAR_CHARARRAY, Tango::DEVVAR_CHARARRAY,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pReadUntilCmd);

	/*----- PROTECTED REGION ID(Socket2Class::command_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		Socket2Class::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the created attribute list
 */
//--------------------------------------------------------
void Socket2Class::create_static_attribute_list(std::vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		std::string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	TANGO_LOG_INFO << defaultAttList.size() << " attributes in default list" << std::endl;

	/*----- PROTECTED REGION ID(Socket2Class::create_static_att_list) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		Socket2Class::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void Socket2Class::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, std::vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((std::string)(*devlist_ptr)[i]).c_str());
		Socket2 *dev = static_cast<Socket2 *> (dev_impl);

		std::vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		std::vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			std::string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			std::vector<std::string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				TANGO_LOG_INFO << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << std::endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(Socket2Class::erase_dynamic_attributes) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	Socket2Class::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method     : Socket2Class::get_attr_object_by_name()
 *	Description: returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *Socket2Class::get_attr_object_by_name(std::vector<Tango::Attr *> &att_list, std::string attname)
{
	std::vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(Socket2Class::Additional Methods) ENABLED START -----*/

/*----- PROTECTED REGION END -----*/	//	Socket2Class::Additional Methods
} //	namespace
